package jp.co.sra.jun.goodies.calendar;

import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import jp.co.sra.smalltalk.StSimpleDialogViewSwing;
import jp.co.sra.smalltalk.WindowSpecLayout;
import jp.co.sra.jun.system.support.JunSystem;

/**
 * JunCalendarDateAndTimeDialogViewSwing class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   N/A
 *  @version   699 (with StPL8.9) based on Jun472 for Smalltalk
 *  @copyright 1999-2008 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2008 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarDateAndTimeDialogViewSwing.java,v 8.12 2008/02/20 06:31:12 nisinaka Exp $
 */
public class JunCalendarDateAndTimeDialogViewSwing extends StSimpleDialogViewSwing implements JunCalendarDateAndTimeView {
	protected JunCalendarDateAndTimeViewSwing dateAndTimeView;
	protected JButton acceptButton;
	protected JButton cancelButton;
	protected JunCalendarDateViewSwing dateView;
	protected JunCalendarTimeViewSwing timeView;

	/**
	 * Create a new instance of <code>JunCalendarDateAndTimeDialogViewSwing</code> and initialize it.
	 * 
	 * @param aCalendarDateAndTimeDialog JunCalendarDateAndTimeDialog
	 * @category Instance creation
	 */
	public JunCalendarDateAndTimeDialogViewSwing(JunCalendarDateAndTimeDialog aCalendarDateAndTimeDialog) {
		super(aCalendarDateAndTimeDialog);
	}

	/**
	 * Answer the default dialog.
	 *
	 * @return java.awt.Dialog
	 * @category defaults
	 */
	public Dialog defaultDialog() {
		Dialog aDialog = new Dialog(new Frame(), this.getCalendarDateAndTimeDialog().windowTitle(), true);
		aDialog.add(this);
		aDialog.pack();
		return aDialog;
	}

	/**
	 * Answer my model as a JunCalendarDateAndTimeDialog.
	 * @return JunCalendarDateAndTimeDialog
	 * @category accessing
	 */
	public JunCalendarDateAndTimeDialog getCalendarDateAndTimeDialog() {
		return (JunCalendarDateAndTimeDialog) this.model();
	}

	/**
	 * Answer my accept button.
	 * @return java.awt.JButton
	 * @category actions
	 */
	protected JButton acceptButton() {
		if (acceptButton == null) {
			acceptButton = new JButton(JunSystem.$String("Accept"));
			acceptButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (((Boolean) getCalendarDateAndTimeDialog().acceptButtonPressed()).booleanValue()) {
						getCalendarDateAndTimeDialog().accept().value_(true);
					}
				}
			});
		}
		return acceptButton;
	}

	/**
	 * Build this component.
	 * @category interface opening
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.dateView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 0f, 300, 1f, -1));
		this.add(this.timeView(), WindowSpecLayout.Position(0f, 300, 0f, 1, 1f, -102, 1f, -1));
		this.add(this.cancelButton(), WindowSpecLayout.Position(1f, -96, 0f, 56, 1f, -8, 0f, 88));
		this.add(this.acceptButton(), WindowSpecLayout.Position(1f, -96, 0f, 16, 1f, -8, 0f, 48));
		defaultButton = this.acceptButton();
		this.setPreferredSize(new Dimension(550, 200));
	}

	/**
	 * Answer my cancel button.
	 *
	 * @return java.awt.JButton
	 * @category actions
	 */
	protected JButton cancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton(JunSystem.$String("Cancel"));
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (((Boolean) getCalendarDateAndTimeDialog().cancelButtonPressed()).booleanValue()) {
						getCalendarDateAndTimeDialog().cancel().value_(true);
					}
				}
			});
		}
		return cancelButton;
	}

	/**
	 * Answer date view.
	 *
	 * @return JunCalendarDateViewSwing
	 * @category accessing
	 */
	protected JunCalendarDateViewSwing dateView() {
		if (dateView == null) {
			dateView = new JunCalendarDateViewSwing(this.getCalendarDateAndTimeDialog().calendarDateAndTimeModel().calendarDateModel());
		}
		return dateView;
	}

	/**
	 * Answer time view.
	 *
	 * @return JunCalendarTimeViewSwing
	 * @category accessing
	 */
	protected JunCalendarTimeViewSwing timeView() {
		if (timeView == null) {
			timeView = new JunCalendarTimeViewSwing(this.getCalendarDateAndTimeDialog().calendarDateAndTimeModel().calendarTimeModel());
		}
		return timeView;
	}
}
