package jp.co.sra.jun.graphics.navigator;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Window;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JLabel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;

import jp.co.sra.smalltalk.DependentEvent;
import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StSymbol;
import jp.co.sra.smalltalk.WindowSpecLayout;

import jp.co.sra.jun.goodies.button.JunButtonViewSwing;
import jp.co.sra.jun.graphics.framework.JunGraphicViewSwing;
import jp.co.sra.jun.graphics.list.JunSequenceListViewSwing;

/**
 * JunFileNavigatorViewSwing class
 * 
 *  @author    nisinaka
 *  @created   2004/01/23 (by nisinaka)
 *  @updated   2006/11/29 (by m-asada)
 *  @version   699 (with StPL8.9) based on Jun638 for Smalltalk
 *  @copyright 1999-2008 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2008 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunFileNavigatorViewSwing.java,v 8.9 2008/02/20 06:32:16 nisinaka Exp $
 */
public class JunFileNavigatorViewSwing extends JunGraphicViewSwing implements JunFileNavigatorView {
	protected JunSequenceListViewSwing centerView;
	protected JunSequenceListViewSwing leftView;
	protected JunSequenceListViewSwing rightView;
	protected JunButtonViewSwing homeButtonView;
	protected JLabel currentFileName;
	protected JLabel statusLabel;

	/**
	 * Create a new instance of <code>JunFileNavigatorViewSwing</code> and initialize it.
	 * 
	 * @param aModel jp.co.sra.jun.graphics.navigator.JunFileNavigator
	 * @category Instance creation
	 */
	public JunFileNavigatorViewSwing(JunFileNavigator aModel) {
		super(aModel);
	}

	/**
	 * Answer the receiver's model as <code>JunFileNavigator</code>.
	 * 
	 * @return jp.co.sra.jun.graphics.navigator.JunFileNavigator
	 * @see jp.co.sra.jun.graphics.navigator.JunFileNavigatorView#getFileNavigator()
	 * @category accessing 
	 */
	public JunFileNavigator getFileNavigator() {
		return (JunFileNavigator) this.model();
	}

	/**
	 * Build the component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#buildComponent()
	 * @category initialize-release
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.homeButtonView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 0f, 25, 0f, 25));
		this.add(this.currentFileName(), WindowSpecLayout.Position(0f, 26, 0f, 1, 1f, -1, 0f, 25));
		this.add(this.leftView(), WindowSpecLayout.Position(0f, 1, 0f, 26, 0.32f, -1, 1f, -21));
		this.add(this.centerView(), WindowSpecLayout.Position(0.32f, 0, 0f, 26, 0.68f, 0, 1f, -21));
		this.add(this.rightView(), WindowSpecLayout.Position(0.68f, 1, 0f, 26, 1f, -1, 1f, -21));
		this.add(this.statusLabel(), WindowSpecLayout.Position(0f, 1, 1f, -20, 1f, -1, 1f, -1));
		this.setPreferredSize(new Dimension(500, 300));
	}

	/**
	 * Answer the center view.
	 * 
	 * @return jp.co.sra.jun.graphics.graphics.list.JunSequenceListViewSwing
	 * @category user interface
	 */
	protected JunSequenceListViewSwing centerView() {
		if (centerView == null) {
			centerView = new JunEmbeddedFileListViewSwing(this.getFileNavigator().centerList());
		}
		return centerView;
	}

	/**
	 * Answer the left view.
	 * 
	 * @return jp.co.sra.jun.graphics.graphics.list.JunSequenceListViewSwing
	 * @category user interface
	 */
	protected JunSequenceListViewSwing leftView() {
		if (leftView == null) {
			leftView = new JunEmbeddedFileListViewSwing(this.getFileNavigator().leftList());
			leftView.setVisible(!this.getFileNavigator().isLeftEnd());
		}
		return leftView;
	}

	/**
	 * Answer the right view.
	 * 
	 * @return jp.co.sra.jun.graphics.graphics.list.JunSequenceListViewSwing
	 * @category user interface
	 */
	protected JunSequenceListViewSwing rightView() {
		if (rightView == null) {
			rightView = new JunSequenceListViewSwing(this.getFileNavigator().rightList());
		}
		return rightView;
	}

	/**
	 * Answer the home button view.
	 * 
	 * @return jp.co.sra.jun.goodies.button.JunButtonViewSwing
	 * @category user interface
	 */
	protected JunButtonViewSwing homeButtonView() {
		if (homeButtonView == null) {
			homeButtonView = new JunButtonViewSwing(this.getFileNavigator().homeButton());
		}
		return homeButtonView;
	}

	/**
	 * Answer the label for the current file name.
	 * 
	 * @return javax.swing.JLabel
	 * @category user interface
	 */
	protected JLabel currentFileName() {
		if (currentFileName == null) {
			currentFileName = new JLabel();
			currentFileName.setFocusable(false);
			currentFileName.setBorder(new EmptyBorder(0, 10, 0, 0));
			if (this.getFileNavigator().currentFileHolder().value() != null) {
				currentFileName.setText(this.getFileNavigator().currentFileAsString());
			}

			this.getFileNavigator().currentFileHolder().compute_(new StBlockClosure() {
				public Object value_(Object anObject) {
					currentFileName.setText(JunFileNavigatorViewSwing.this.getFileNavigator().currentFileAsString());
					return null;
				}
			});
		}
		return currentFileName;
	}

	/**
	 * Answer the label for the status.
	 * 
	 * @return javax.swing.JLabel
	 * @category user interface
	 */
	protected JLabel statusLabel() {
		if (statusLabel == null) {
			statusLabel = new JLabel(this.getFileNavigator().currentStatus());
			statusLabel.setHorizontalAlignment(SwingConstants.CENTER);
		}
		return statusLabel;
	}

	/**
	 * To fix the scrolling offset when opened.
	 * 
	 * @see java.awt.Component#addNotify()
	 * @category actions
	 */
	public void addNotify() {
		super.addNotify();

		this.topComponent().addWindowListener(new WindowAdapter() {
			public void windowOpened(WindowEvent e) {
				JunFileNavigatorViewSwing.this.getFileNavigator().fixScrollingOffset();
			}
		});
	}

	/**
	 * Set up the keyboard for the view on the window.
	 * 
	 * @param aWindow java.awt.Window
	 * @see jp.co.sra.smalltalk.StView#_setupKeyboard(java.awt.Window)
	 * @category keyboard
	 */
	public void _setupKeyboard(final Window aWindow) {
		KeyListener keyListener = this.centerView().getSequenceList()._keyListener();
		if (keyListener == null) {
			return;
		}

		this.centerView().addKeyListener(keyListener);

		MouseListener mouseListener = new MouseAdapter() {
			public void mouseEntered(MouseEvent e) {
				if (aWindow.isActive() && aWindow.getFocusOwner() != JunFileNavigatorViewSwing.this.centerView()) {
					JunFileNavigatorViewSwing.this.centerView().requestFocus();
				}
			}
		};
		this.centerView().canvas().addMouseListener(mouseListener);
		this.leftView().canvas().addMouseListener(mouseListener);
		this.rightView().canvas().addMouseListener(mouseListener);

		aWindow.addWindowListener(new WindowAdapter() {
			public void windowActivated(WindowEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						Component focusOwner = aWindow.getFocusOwner();
						if (focusOwner == null || focusOwner != JunFileNavigatorViewSwing.this.centerView()) {
							JunFileNavigatorViewSwing.this.centerView().requestFocus();
						}
					}
				});
			}
		});
	}

	/**
	 * Update the receiver according to the change notification from the model.
	 * 
	 * @param evt jp.co.sra.smalltalk.DevendentEvent
	 * @see jp.co.sra.smalltalk.DependentListener#update_(jp.co.sra.smalltalk.DependentEvent)
	 * @category updating
	 */
	public void update_(DependentEvent evt) {
		StSymbol aspect = evt.getAspect();
		if (aspect == $("current file")) {
			this.statusLabel().setText(this.getFileNavigator().currentStatus());
			this.setListViewVisible(this.leftView(), !this.getFileNavigator().isLeftEnd());
			this.setListViewVisible(this.rightView(), !this.getFileNavigator().isCurrentFileInaccessibleDirectory());
		} else {
			super.update_(evt);
		}
	}

	/**
	 * Set the list view visibility.
	 * 
	 * @param listView jp.co.sra.jun.graphics.graphics.list.JunSequenceListViewSwing
	 * @param aFlag boolean
	 * @category private
	 */
	protected void setListViewVisible(JunSequenceListViewSwing listView, boolean aFlag) {
		if (listView.isVisible() != aFlag) {
			listView.setVisible(aFlag);
		}
	}
}
